@@ -1,56 +1,32 @@
 #' HTML book built with bootstrap4
 #'
 #' @description
-#' This output format is built with [bootstrap](https://getbootstrap.com),
-#' using carefully craft features to provide a clean reading experience whether
-#' your on a phone, tablet, or desktop.
-#'
-#' Some of the main features:
-#'
-#' * Easy customisation of colours and fonts with
-#'   [bslib](https://rstudio.github.io/bslib/)
-#'
-#' * Built in search (broken down by section) that helps you quickly find what
-#'   you're looking for.
-#'
-#' * A sidebar containing a within-chapter table of contents that makes
-#'   navigation easy and helps provide context about your current position
-#'   within the chapter.
-#'
-#' * Thoughtful typography to make the contents as easy as possible to read,
-#'   regardless of the size of your device. A sticky header gets out of your
-#'   way when reading, but is easily accessible if you need it.
-#'
-#' * In-line footnotes mean you can read asides next to the next they refer
-#'   to. This theme is best paired with a reference style that generates
-#'   footnotes.
-#'
-#' * R syntax highlighting and autolinking by
-#'   [downlit](https://downlit.r-lib.org/) is paired with a accessible
-#'   colour scheme designed by Alison Hill.
-#'
-#' This theme is designed for books that use one chapter per page.
+#' This output format is built with [Bootstrap](https://getbootstrap.com),
+#' using carefully crafted features to provide a clean reading experience whether
+#' you are on a phone, tablet, or desktop. To read more about this format, see:
+#' \url{https://bookdown.org/yihui/bookdown/html.html#bootstrap4-style}
 #'
 #' @param theme A named list or [bslib::bs_theme()] object.
 #'   The default, `bs4_book_theme()`, resets the base font size to 1rem to
-#'   make reading easier and uses a primary colour with greater constrast
+#'   make reading easier and uses a primary colour with greater contrast
 #'   against the background.
-#' @param repo Link to repository where book is hosted, used to generate
-#'   view source and edit buttons. Currently assumes GitHub and that the book
-#'   is in the root directory of the repo.
-#' @param lib_dir,pandoc_args,extra_dependencies,... Passed on to
+#' @param repo Either link to repository where book is hosted, used to generate
+#'   view source and edit buttons or a list with repository `base` link, default
+#'   `branch`, `subdir` and `icon`
+#'   (see "Specifying the repository" in \url{https://bookdown.org/yihui/bookdown/html.html#bootstrap4-style}).
+#' @param lib_dir,pandoc_args,extra_dependencies,split_bib,... Passed on to
 #'   [rmarkdown::html_document()].
+#'
 #' @export
 #' @md
-bs4_book <- function(
-                     theme = bs4_book_theme(),
+bs4_book <- function(theme = bs4_book_theme(),
                      repo = NULL,
                      ...,
                      lib_dir = "libs",
                      pandoc_args = NULL,
-                     extra_dependencies = NULL
-                     ) {
-  check_packages(c("bslib", "downlit", "jsonlite", "xml2"))
+                     extra_dependencies = NULL,
+                     split_bib = FALSE) {
+  check_packages(bs4_book_deps())
   bs4_check_dots(...)
 
   # Allow theme specification in yaml metadata
@@ -77,7 +53,8 @@ bs4_book <- function(
       output <- post(metadata, input, output, clean, verbose)
     }
 
-    output2 <- bs4_book_build(output, repo = repo, lib_dir = lib_dir)
+    output2 <- bs4_book_build(output, repo = repo, lib_dir = lib_dir,
+                              split_bib = split_bib)
 
     if (clean && file.exists(output) && !same_path(output, output2)) {
       file.remove(output)
@@ -92,8 +69,12 @@ bs4_book <- function(
 #' @export
 #' @rdname bs4_book
 #' @param primary Primary colour: used for links and background of footer.
-bs4_book_theme <- function(primary = "#0068D9", ...) {
+#' @param version Passed to [bslib::bs_theme()]. This should not be changed as
+#'  [bs4_book()] has been designed to work with Bootstrap version 4 for now.
+#' @md
+bs4_book_theme <- function(primary = "#0068D9", version = 4, ...) {
   bslib::bs_theme(...,
+    version = 4,
     primary = primary,
     "font-size-base" = "1rem",
   )
@@ -102,17 +83,17 @@ bs4_book_theme <- function(primary = "#0068D9", ...) {
 bs4_book_build <- function(output = "bookdown.html",
                            repo = NULL,
                            lib_dir = "libs",
-                           output_dir = opts$get("output_dir")
-                           ) {
+                           output_dir = opts$get("output_dir"),
+                           split_bib = split_bib) {
   move_files_html(output, lib_dir)
 
   rmd_index <- new.env(parent = emptyenv())
   output2 <- split_chapters(
     output = output,
     build = function(...) bs4_book_page(..., rmd_index = rmd_index),
-    number_sections = TRUE,
+    global_numbering = FALSE,
     split_by = "chapter",
-    split_bib = FALSE
+    split_bib = split_bib
   )
 
   move_files_html(output2, lib_dir)
@@ -121,7 +102,7 @@ bs4_book_build <- function(output = "bookdown.html",
     output_dir <- "_book"
   }
 
-  if (isTRUE(opts$get('preview'))) {
+  if (isTRUE(opts$get("preview"))) {
     bs4_chapter_tweak(
       output2,
       repo = repo,
@@ -179,8 +160,8 @@ build_toc <- function(output) {
   if (any(is_appendix)) {
     app <- toc[
       seq_along(is_appendix) > which(is_appendix)[[1]] &
-      toc$level == 1 &
-      !is.na(toc$num),
+        toc$level == 1 &
+        !is.na(toc$num),
     ]
     app$label <- LETTERS[seq_len(nrow(app))]
     # TODO: make less of a hack
@@ -202,17 +183,17 @@ build_toc <- function(output) {
   toc
 }
 
-bs4_book_page = function(head,
-                         toc,
-                         chapter,
-                         link_prev,
-                         link_next,
-                         rmd_cur,
-                         html_cur,
-                         foot,
-                         rmd_index = NULL) {
+bs4_book_page <- function(head,
+                          toc,
+                          chapter,
+                          link_prev,
+                          link_next,
+                          rmd_cur,
+                          html_cur,
+                          foot,
+                          rmd_index = NULL) {
   rmd_index[[html_cur]] <- rmd_cur
-  paste(c(head, toc, chapter, foot), collapse = '\n')
+  paste(c(head, toc, chapter, foot), collapse = "\n")
 }
 
 bs4_book_dependency <- function(theme) {
@@ -239,7 +220,6 @@ bs4_chapters_tweak <- function(output,
                                rmd_index = NULL,
                                repo = NULL,
                                output_dir = opts$get("output_dir")) {
-
   toc <- build_toc(output)
   files <- toc[!duplicated(toc$file_name) & !is.na(toc$file_name), ]
   files$path <- file.path(output_dir, files$file_name)
@@ -250,6 +230,13 @@ bs4_chapters_tweak <- function(output,
     message("Tweaking ", path)
     index[[i]] <- bs4_chapter_tweak(path, toc, rmd_index = rmd_index, repo = repo)
   }
+  # tweak 404.html ---
+  path_404 <- file.path(output_dir, "404.html")
+  if (file.exists(path_404)) {
+    message("Tweaking ", path_404)
+    bs4_chapter_tweak(path_404, toc, rmd_index = rmd_index, repo = repo)
+
+  }
   index <- unlist(index, recursive = FALSE, use.names = FALSE)
 
   jsonlite::write_json(
@@ -264,6 +251,7 @@ bs4_chapter_tweak <- function(path, toc, rmd_index = NULL, repo = NULL) {
 
   # Convert ANSI escape to \u2029 since control characters are ignored in XML2
   text <- gsub("\033", "&#8233;", text, fixed = TRUE, useBytes = TRUE)
+  Encoding(text) <- "UTF-8" # gsub with useBytes inhibits marked encoding
   html <- xml2::read_html(text, encoding = "UTF-8")
 
   tweak_tables(html)
@@ -273,6 +261,8 @@ bs4_chapter_tweak <- function(path, toc, rmd_index = NULL, repo = NULL) {
   tweak_footnotes(html)
   tweak_part_screwup(html)
   tweak_navbar(html, toc, basename(path), rmd_index = rmd_index, repo = repo)
+  tweak_metadata(html, path)
+  if (basename(path) == "404.html") tweak_404(html)
   downlit::downlit_html_node(html)
 
   xml2::write_html(html, path, format = FALSE)
@@ -283,7 +273,11 @@ bs4_chapter_tweak <- function(path, toc, rmd_index = NULL, repo = NULL) {
     chapter = h1,
     path = basename(path)
   )
+}
 
+tweak_404 <- function(html) {
+  sidebar_toc <- xml2::xml_find_all(html, ".//div[contains(@class, 'sidebar')]/nav[@id='toc']")
+  xml2::xml_remove(sidebar_toc)
 }
 
 tweak_chapter <- function(html) {
@@ -316,7 +310,7 @@ tweak_footnotes <- function(html) {
   id <- xml2::xml_attr(footnotes, "id")
   xml2::xml_remove(xml2::xml_find_all(footnotes, "//a[@class='footnote-back']"))
   contents <- vapply(footnotes, FUN.VALUE = character(1), function(x) {
-    as.character(xml2::xml_children(x))
+    paste0(as.character(xml2::xml_children(x)), collapse = "")
   })
 
   # Add popover attributes to links
@@ -390,17 +384,36 @@ tweak_tables <- function(html) {
 }
 
 tweak_navbar <- function(html, toc, active = "", rmd_index = NULL, repo = NULL) {
+  if (!is.null(repo) && length(repo) == 1) {
+    repo <- list(
+      base = repo,
+      branch = "master",
+      subdir = NULL
+    )
+  }
 
   # Source links ------------------------------------------------------------
   if (!is.null(repo) && active %in% names(rmd_index)) {
-    repo_edit <- paste0(repo, "/edit/master/", rmd_index[[active]])
-    repo_view <- paste0(repo, "/blob/master/", rmd_index[[active]])
+    if (!is.null(repo$subdir)) {
+      repo$subdir <- paste0(repo$subdir, "/")
+    }
+
+    repo_edit <- paste0(repo$base, "/edit/", repo$branch, "/", repo$subdir, rmd_index[[active]])
+    repo_view <- paste0(repo$base, "/blob/", repo$branch, "/", repo$subdir, rmd_index[[active]])
   } else {
     repo_edit <- NULL
     repo_view <- NULL
   }
 
-  template_link(html, ".//a[@id='book-repo']", repo)
+  if (!is.null(repo$base)) {
+    icon <- repo$icon %n%
+      ifelse(grepl("github\\.com", repo$base), "fab fa-github", "fab fa-gitlab")
+    template_link_icon(html, ".//a[@id='book-repo']", icon)
+    template_link_icon(html, ".//a[@id='book-source']", icon)
+    template_link_icon(html, ".//a[@id='book-edit']", icon)
+  }
+
+  template_link(html, ".//a[@id='book-repo']", repo$base)
   template_link(html, ".//a[@id='book-source']", repo_view)
   template_link(html, ".//a[@id='book-edit']", repo_edit)
 
@@ -516,10 +529,80 @@ template_link <- function(html, xpath, href) {
   }
 }
 
+tweak_metadata <- function(html, path) {
+  file <- basename(path)
+
+  # Fix generator
+  generator <- xml_find_meta_name(html, 'generator')
+  bookdown_string <- sprintf("bookdown %s with bs4_book()", packageVersion("bookdown"))
+  set_content(generator, bookdown_string)
+
+  # Check there are descriptions, add them if not
+  general_description <- xml_find_meta_name(html, 'description')
+
+  # Add the nodes if they were missing by default
+  if (length(general_description) == 0) {
+    head <- xml2::xml_find_first(html, '//head')
+    default_description <- "A book created with bookdown."
+    if (file == "index.html") message("TODO: Add a description field in the YAML metadata of index.Rmd.")
+    xml2 <- xml2::xml_add_child(head, "meta", name= "description", content = default_description)
+    xml2 <- xml2::xml_add_child(head, "meta", property = "og:description", content = default_description)
+    xml2 <- xml2::xml_add_child(head, "meta", name = "twitter:description", content = default_description)
+  }
+
+  # index page is the only one not modified - yaml provided description by user is used.
+  if (file == "index.html") return(invisible())
+
+  # Fix og:url
+  og_url <- xml_find_meta_property(html, 'og:url')
+  base_url <- xml2::xml_attr(og_url, "content")
+   if (!grepl("/$", base_url)) base_url <- paste0(base_url, "/")
+  set_content(og_url, paste0(base_url, file))
+
+  # Fix descriptions if possible
+  general_description <- xml_find_meta_name(html, 'description')
+  twitter_description <- xml_find_meta_name(html, 'twitter:description')
+  og_description <- xml_find_meta_property(html, 'og:description')
+
+  contents <- copy_html(xml2::xml_find_first(html, "//main[@id='content']"))
+  xml2::xml_remove(xml2::xml_find_first(contents, "//h1"))
+  xml2::xml_remove(xml2::xml_find_first(contents, "//div[@class='chapter-nav']"))
+  text <- xml2::xml_text(contents)
+  text <- gsub("\\\n", " ", text)
+  text <- gsub("  ", " ", text)
+  text <- gsub("^[[:space:]]+", "", text)
+  text <- gsub("[[:space:]]+$", "", text)
+  if (nzchar(text)) {
+    words <- unlist(strsplit(text, " "))
+    no_char <- cumsum(unlist(lapply(words, function(x) {nchar(x) + 1})))
+    max_n <- max(which(no_char<= 197))
+    description_string <- paste(words[1: max_n], collapse = " ")
+    if (max_n != length(words)) {
+      description_string <- paste0(description_string, "...")
+    }
+    set_content(og_description, description_string)
+    set_content(twitter_description, description_string)
+    set_content(general_description, description_string)
+  }
+
+}
+
+# https://github.com/ropensci/tinkr/blob/935ed21439230228f07f26161a507812d0fc76c3/R/to_md.R#L68
+# TODO: replace by xml_clone() when it exists (https://github.com/r-lib/xml2/issues/341)
+copy_html <- function(html) {
+  xml2::read_html(as.character(html))
+}
+
+template_link_icon <- function(html, xpath, icon) {
+  icon_node <- xml2::xml_child(xml2::xml_find_first(html, xpath))
+  xml2::xml_attr(icon_node, "class") <- icon
+}
+
 # index -------------------------------------------------------------------
 
 bs4_index_data <- function(node, chapter, path) {
-  children <- xml2::xml_find_all(node,
+  children <- xml2::xml_find_all(
+    node,
     "./*[not(self::div and contains(@class, 'section'))]"
   )
   if (length(children) == 0 || !is_heading(children[[1]])) {
@@ -527,7 +610,7 @@ bs4_index_data <- function(node, chapter, path) {
   }
 
   all <- function(...) paste0("descendant-or-self::", c(...), collapse = "|")
-  text_path <- all("p", "li", "caption", "figcaption", "dt", "dd")
+  text_path <- all("p", "li", "caption", "figcaption", "dt", "dd", "div[contains(@class, 'line-block')]")
   code_path <- all("pre")
 
   code <- xml2::xml_find_all(children, code_path)
@@ -616,3 +699,22 @@ bs4_check_dots <- function(...) {
     )
   }
 }
+
+set_content <- function(node, content) {
+  xml2::xml_set_attr(node, "content", content)
+}
+
+xml_find_meta_property <- function(html, property){
+    xml2::xml_find_first(html, sprintf('//meta[@property="%s"]', property))
+}
+
+xml_find_meta_name <- function(html, property){
+    xml2::xml_find_first(html, sprintf('//meta[@name="%s"]', property))
+}
+
+# these dependencies are required to use bs4_book() but are suggested deps
+# of bookdown. Hence the need to check they are available
+# TODO: remove this and the check in bs4_book when we add them as Imports (if we do it)
+bs4_book_deps <- function() {
+  c("bslib", "downlit", "jsonlite", "xml2")
+}
